==> apiconnect-cli-util-1.0.3.tgz: package/lib/project/validator.js <==
// Copyright IBM Corp. 2016. All Rights Reserved.
// Node module: apiconnect-cli-util
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

/********************************************************* {COPYRIGHT-TOP} ***
 * IBM Confidential
 * OCO Source Materials
 * 5725-C11, 5725-C12, 7198-8FX, 5725-F70, 5725-F71, 5725-F72, 5725-L30
 *
 * (C) Copyright IBM Corporation 2015  All Rights Reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 ********************************************************* {COPYRIGHT-END} **/
/*eslint brace-style: 0*/

/**
 * A project entity validator that validates against a json schema
 * but also allows to add specific validation rules.
 */
var tv4 = require('tv4');
var _ = require('lodash');
var logger = require('apiconnect-cli-logger');

var JSON_SCHEMA_SWAGGER_X_IBM;
var JSON_SCHEMA_SWAGGER_X_IBM_CONFIGURATION;
var JSON_SCHEMA_PRODUCT_IBM;

var SWAGGER_EXTENSION_CONFIGURATION = 'x-ibm-configuration';
var SWAGGER_EXTENSION_CONFIGURATION_ENFORCED = 'enforced';
var SWAGGER_EXTENSION_CONFIGURATION_PHASE = 'phase';
var SWAGGER_EXTENSION_CONFIGURATION_PHASE_REALIZED = 'realized';
var RESPONSE_EXTENSION_PATTERN = '^x-.+$';
var JSON_REF = '$ref';
var SWAGGER_2_API_SCHEMES_NOT_VALID = 'Swagger 2 API Schemes is not valid';
var SWAGGER_2_API_SCHEMES_MISSING_PROTOCOL =
      'Swagger 2 API Schemes supports only https as protocol';
var SWAGGER_2_API_HOST_MISSING = 'Swagger 2 API host missing';
var SWAGGER_JSON_REF_NOT_SUPPORTED = 'Swagger Json Ref is not supported';
var VALIDATOR_TYPES = { api: true, product: true };
var SUPPORTED_OPERATIONS = { get: true, put: true, post: true, delete: true };

var vendorExtensions;
var validatorType;
var apimError = [];

/**
 * All Validators instances gets registered and stored here.
 *
 * @type {'<type>': Validator}    where type is 'api', 'product'
 */
var validators = {};
var mainSchema = {};
var extraValidators = {};

function Validator() {
  JSON_SCHEMA_SWAGGER_X_IBM =
    require('../../resources/swagger-schema-v2.0-ibm.json');
  JSON_SCHEMA_SWAGGER_X_IBM_CONFIGURATION =
    require('../../resources/swagger-v2.0-x-ibm-configuration.json');
  JSON_SCHEMA_PRODUCT_IBM =
    require('../../resources/product-schema.json');
  mainSchema.api = JSON_SCHEMA_SWAGGER_X_IBM;
  mainSchema.product = JSON_SCHEMA_PRODUCT_IBM;
  this.schemas = {};
}
/**
 * Create a validator instance that validates a swagger api according to
 * the extended swagger definition.
 *
 * @return {@see Validator}
 */
var createApiValidator = function() {
  var validator = new Validator();
  validator.addSchema('JSON_SCHEMA_SWAGGER_X_IBM',
    JSON_SCHEMA_SWAGGER_X_IBM);
  validator.addSchema('JSON_SCHEMA_SWAGGER_X_IBM_CONFIGURATION',
    JSON_SCHEMA_SWAGGER_X_IBM_CONFIGURATION);
  tv4.addSchema(JSON_SCHEMA_SWAGGER_X_IBM);
  tv4.addSchema(JSON_SCHEMA_SWAGGER_X_IBM_CONFIGURATION);
  return validator;
};

/**
 * Create a Validator instance that validates a product type
 * @return {@see Validator}
 */
var createProductValidator = function() {
  var validator = new Validator();
  validator.addSchema('JSON_SCHEMA_PRODUCT_IBM', JSON_SCHEMA_PRODUCT_IBM);
  return validator;
};
validators.api = createApiValidator;
validators.product = createProductValidator;

Validator.prototype.addSchema = function(ns, schema) {
  this.schemas[ns] = schema;
};
/**
 * Return an object of this format {valid: true|false, error: error}
 *
 * @param data
 * @returns {object}
 */
Validator.prototype.validate = function(data) {
  var schema = mainSchema[validatorType];
  var validators = extraValidators[validatorType];

  try {
    var result = tv4.validateMultiple(data, schema);

    var i = 0;
    for (i = 0; i < validators.length; i++) {
      validators[i].apply(this, [ data ]);
    }

    if (result.valid === false || apimError.length > 0) {
      return {
        valid: false,
        error: {
          schemaErrors: result,
          apimErrors: apimError,
        },
      };
    }

    return { valid: true };
  }
  catch (ex) {
    logger.writeln('Exception: ' + ex);
    return { valid: false, error: ex };
  }
};

function validateApim(data) {
  if (isRealized() && isEnforced()) {
    validateSchemesForApimLimitations(data);
    validateSecurityForApimLimitations(data);
  }

  if (isRealized() && !isEnforced()) {
    validateHostForNonApimLimitations();
  }
}

function validateResource(data) {
  if (isRealized() && isEnforced()) {
    validateSchemesForApimLimitations(data);  // Same for APIM, running it once.
    validateResponsesForAPIMLimitations(data);
    validateParametersForAPIMLimitations(data);
    validateSecurityForAPIMLimitations(data);
  }
}

validators.api = createApiValidator;
validators.product = createProductValidator;
extraValidators.api = [ validateApim, validateResource ];
extraValidators.product = [];

function validateSchemesForApimLimitations(data) {
  var schemes = data.schemes;

  if (!schemes) {
    schemes = {};
    schemes.https = true;
  }

  if (_.size(schemes) === 0) {
    apimError.push(SWAGGER_2_API_SCHEMES_NOT_VALID);
    return;
  }

  if (_.size(schemes) > 1 || _.indexOf(schemes, 'https') > 0) {
    apimError.push(SWAGGER_2_API_SCHEMES_MISSING_PROTOCOL);
    return;
  }
}

/**
 * Validating responses for current APIM limitations.
 * Currently responses can not have $ref
 *
 * @param data swagger yml file to validate
 */
function validateResponsesForAPIMLimitations(data) {
  if (!data) {
    return;
  }
  Object.keys(data.paths).forEach(function(pathKey) {
    var path = data.paths[pathKey];
    Object.keys(path).forEach(function(operationKey) {
      if (operationKey in SUPPORTED_OPERATIONS) {
        var responses = path[operationKey].responses;
        Object.keys(responses).forEach(function(responseKey) {
          if (responseKey.search(RESPONSE_EXTENSION_PATTERN) === -1) {
            var response = responses[responseKey];

            Object.keys(response).forEach(function(key) {
              if (key === JSON_REF) {
                apimError.push(SWAGGER_JSON_REF_NOT_SUPPORTED + ' Path: '
                  + pathKey + ' Operation: ' + operationKey +
                  ' Response: ' + responseKey);
                //no need to return here, we want to be able to validate
                // errors in multiple responses, and not necessarily
                // stop on the first
              }
            });
          }
        });
      }
    });
  });
}

/**
 * Validating parameters for current APIM limitations.
 * Currently parameters can not have $ref
 *
 * @param data swagger yml file to validate
 */
function validateParametersForAPIMLimitations(data) {
  if (!data) {
    return;
  }
  Object.keys(data.paths).forEach(function(pathKey) {
    var path = data.paths[pathKey];
    Object.keys(path).forEach(function(operationKey) {
      if (operationKey in SUPPORTED_OPERATIONS) {
        var parameters = path[operationKey].parameters;
        if (!parameters) {
          return;
        }
        Object.keys(parameters).forEach(function(parameterKey) {
          var parameter = parameters[parameterKey];

          Object.keys(parameter).forEach(function(key) {
            if (key === JSON_REF) {
              apimError.push(
                SWAGGER_JSON_REF_NOT_SUPPORTED + ' Path: ' + pathKey +
                ' Operation: ' + operationKey + ' Parameter: ' + parameterKey);
              //no need to return here, we want to be able to validate errors
              // in multiple parameters, and not necessarily stop on the first
            }
          });
        });
      }
    });
  });

  // Todo: load the parameters of the operations and
  // then make sure there are no JSON_REF in them
  var parameters = null;

  //validate the parameters
  if (parameters === null || parameters.length() === 0) {
    return;
  }

  for (var i = 0; i < parameters.length(); i++) {
    var field = getSwaggerFieldPath() + '.parameters[' + i + ']';
    var param = parameters[i];

    if (JSON_REF in param) {
      apimError.push(SWAGGER_JSON_REF_NOT_SUPPORTED + field);
      return;
    }
  }
}

function getSwaggerFieldPath() {
  throw new Error('Not implemented yet!');
}

function validateSecurityForAPIMLimitations(data) {
  // Todo: If needed for off line validation
}

function validateSecurityForApimLimitations(data) {
  // Todo: If needed for off line validation
}

function validateHostForNonApimLimitations(data) {
  if (_.trim(data) === null) {
    logger.error(SWAGGER_2_API_HOST_MISSING);
  }
}

function isEnforced() {
  // default 'enforced' is true
  // Todo: load vendor extensions from the yml file incase specified
  if (!vendorExtensions) {
    return true;
  }

  var configuration = vendorExtensions[SWAGGER_EXTENSION_CONFIGURATION];
  if (!configuration) {
    return true;
  }

  var isEnforcedVar = configuration[SWAGGER_EXTENSION_CONFIGURATION_ENFORCED];
  if (isEnforcedVar === null || isEnforcedVar === true) {
    return true;
  }

  return false;
}

function isRealized() {
  // default 'realized' is true
  var phase = getPhase();
  if (phase === null) {
    return true;
  }

  return phase.toLowerCase() === SWAGGER_EXTENSION_CONFIGURATION_PHASE_REALIZED;
}

function getPhase() {
  // Todo: load vendor extensions from the yaml file incase specified
  if (!vendorExtensions) {
    return null;
  }

  var configuration = vendorExtensions[SWAGGER_EXTENSION_CONFIGURATION];
  if (!configuration) {
    return null;
  }

  var phase = configuration[SWAGGER_EXTENSION_CONFIGURATION_PHASE];
  return _.trim(phase);
}

/**
 * Create a {@see Validator} instance based on {@see type}
 *
 * @param type {string} valid values {'api', 'type'}
 * @return {@see Validator}
 * @throw Error if no validate has been found
 */
module.exports.createValidator = function(/**String*/type) {
  if (!(type in VALIDATOR_TYPES)) {
    logger.writeln('Invalid validator type : ' + type);
    throw new Error('Invalid validator type!');
    //return;
  }

  validatorType = type;

  if (type && validators[type]) {
    return validators[type]();
  }
  else {
    return new Validator();   //Todo: Is this a valid scenario ?
  }
};

==> apiconnect-cli-util-3.0.3.tgz: package/lib/project/validator.js <==
// Copyright IBM Corp. 2016. All Rights Reserved.
// Node module: apiconnect-cli-util
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

/********************************************************* {COPYRIGHT-TOP} ***
 * IBM Confidential
 * OCO Source Materials
 * 5725-C11, 5725-C12, 7198-8FX, 5725-F70, 5725-F71, 5725-F72, 5725-L30
 *
 * (C) Copyright IBM Corporation 2015  All Rights Reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 ********************************************************* {COPYRIGHT-END} **/
/*eslint brace-style: 0*/

/**
 * A project entity validator that validates against a json schema
 * but also allows to add specific validation rules.
 */
var tv4 = require('tv4');
var _ = require('lodash');
var f = require('util').format;
var logger = require('apiconnect-cli-logger');

var JSON_SCHEMA_SWAGGER_X_IBM;
var JSON_SCHEMA_SWAGGER_X_IBM_CONFIGURATION;
var JSON_SCHEMA_PRODUCT_IBM;

var SWAGGER_EXTENSION_CONFIGURATION = 'x-ibm-configuration';
var SWAGGER_EXTENSION_CONFIGURATION_ENFORCED = 'enforced';
var SWAGGER_EXTENSION_CONFIGURATION_PHASE = 'phase';
var SWAGGER_EXTENSION_CONFIGURATION_PHASE_REALIZED = 'realized';
var RESPONSE_EXTENSION_PATTERN = '^x-.+$';
var JSON_REF = '$ref';
var SWAGGER_2_API_SCHEMES_NOT_VALID = 'Swagger 2 API Schemes is not valid';
var SWAGGER_2_API_SCHEMES_MISSING_PROTOCOL =
      'Swagger 2 API Schemes supports only https as protocol';
var SWAGGER_2_API_HOST_MISSING = 'Swagger 2 API host missing';
var SWAGGER_JSON_REF_NOT_SUPPORTED = 'Swagger Json Ref is not supported';
var VALIDATOR_TYPES = { api: true, product: true };
var SUPPORTED_OPERATIONS = { get: true, put: true, post: true, delete: true };

var vendorExtensions;
var validatorType;
var apimError = [];

/**
 * All Validators instances gets registered and stored here.
 *
 * @type {'<type>': Validator}    where type is 'api', 'product'
 */
var validators = {};
var mainSchema = {};
var extraValidators = {};

function Validator() {
  JSON_SCHEMA_SWAGGER_X_IBM =
    require('../../resources/swagger-schema-v2.0-ibm.json');
  JSON_SCHEMA_SWAGGER_X_IBM_CONFIGURATION =
    require('../../resources/swagger-v2.0-x-ibm-configuration.json');
  JSON_SCHEMA_PRODUCT_IBM =
    require('../../resources/product-schema.json');
  mainSchema.api = JSON_SCHEMA_SWAGGER_X_IBM;
  mainSchema.product = JSON_SCHEMA_PRODUCT_IBM;
  this.schemas = {};
}
/**
 * Create a validator instance that validates a swagger api according to
 * the extended swagger definition.
 *
 * @return {@see Validator}
 */
var createApiValidator = function() {
  var validator = new Validator();
  validator.addSchema('JSON_SCHEMA_SWAGGER_X_IBM',
    JSON_SCHEMA_SWAGGER_X_IBM);
  validator.addSchema('JSON_SCHEMA_SWAGGER_X_IBM_CONFIGURATION',
    JSON_SCHEMA_SWAGGER_X_IBM_CONFIGURATION);
  tv4.addSchema(JSON_SCHEMA_SWAGGER_X_IBM);
  tv4.addSchema(JSON_SCHEMA_SWAGGER_X_IBM_CONFIGURATION);
  return validator;
};

/**
 * Create a Validator instance that validates a product type
 * @return {@see Validator}
 */
var createProductValidator = function() {
  var validator = new Validator();
  validator.addSchema('JSON_SCHEMA_PRODUCT_IBM', JSON_SCHEMA_PRODUCT_IBM);
  return validator;
};
validators.api = createApiValidator;
validators.product = createProductValidator;

Validator.prototype.addSchema = function(ns, schema) {
  this.schemas[ns] = schema;
};
/**
 * Return an object of this format {valid: true|false, error: error}
 *
 * @param data
 * @returns {object}
 */
Validator.prototype.validate = function(data) {
  var schema = mainSchema[validatorType];
  var validators = extraValidators[validatorType];

  try {
    var result = tv4.validateMultiple(data, schema);

    var i = 0;
    for (i = 0; i < validators.length; i++) {
      validators[i].apply(this, [ data ]);
    }

    if (result.valid === false || apimError.length > 0) {
      return {
        valid: false,
        error: {
          schemaErrors: result,
          apimErrors: apimError,
        },
      };
    }

    return { valid: true };
  }
  catch (ex) {
    logger.writeln('Exception: ' + ex);
    return { valid: false, error: ex };
  }
};

function validateApim(data) {
  if (isRealized() && isEnforced()) {
    validateSchemesForApimLimitations(data);
    validateSecurityForApimLimitations(data);
  }

  if (isRealized() && !isEnforced()) {
    validateHostForNonApimLimitations();
  }
}

function validateResource(data) {
  if (isRealized() && isEnforced()) {
    validateSchemesForApimLimitations(data);  // Same for APIM, running it once.
    validateResponsesForAPIMLimitations(data);
    validateParametersForAPIMLimitations(data);
    validateSecurityForAPIMLimitations(data);
  }
}

validators.api = createApiValidator;
validators.product = createProductValidator;
extraValidators.api = [ validateApim, validateResource ];
extraValidators.product = [];

function validateSchemesForApimLimitations(data) {
  var schemes = data.schemes;

  if (!schemes) {
    schemes = {};
    schemes.https = true;
  }

  if (_.size(schemes) === 0) {
    apimError.push(SWAGGER_2_API_SCHEMES_NOT_VALID);
    return;
  }

  if (_.size(schemes) > 1 || _.indexOf(schemes, 'https') > 0) {
    apimError.push(SWAGGER_2_API_SCHEMES_MISSING_PROTOCOL);
    return;
  }
}

/**
 * Validating responses for current APIM limitations.
 * Currently responses can not have $ref
 *
 * @param data swagger yml file to validate
 */
function validateResponsesForAPIMLimitations(data) {
  if (!data) {
    return;
  }
  Object.keys(data.paths).forEach(function(pathKey) {
    var path = data.paths[pathKey];
    Object.keys(path).forEach(function(operationKey) {
      if (operationKey in SUPPORTED_OPERATIONS) {
        var responses = path[operationKey].responses;
        Object.keys(responses).forEach(function(responseKey) {
          if (responseKey.search(RESPONSE_EXTENSION_PATTERN) === -1) {
            var response = responses[responseKey];

            Object.keys(response).forEach(function(key) {
              if (key === JSON_REF) {
                apimError.push(SWAGGER_JSON_REF_NOT_SUPPORTED + ' Path: '
                  + pathKey + ' Operation: ' + operationKey +
                  ' Response: ' + responseKey);
                //no need to return here, we want to be able to validate
                // errors in multiple responses, and not necessarily
                // stop on the first
              }
            });
          }
        });
      }
    });
  });
}

/**
 * Validating parameters for current APIM limitations.
 * Currently parameters can not have $ref
 *
 * @param data swagger yml file to validate
 */
function validateParametersForAPIMLimitations(data) {
  if (!data) {
    return;
  }
  Object.keys(data.paths).forEach(function(pathKey) {
    var path = data.paths[pathKey];
    Object.keys(path).forEach(function(operationKey) {
      if (operationKey in SUPPORTED_OPERATIONS) {
        var parameters = path[operationKey].parameters;
        if (!parameters) {
          return;
        }
        Object.keys(parameters).forEach(function(parameterKey) {
          var parameter = parameters[parameterKey];

          Object.keys(parameter).forEach(function(key) {
            if (key === JSON_REF) {
              apimError.push(
                SWAGGER_JSON_REF_NOT_SUPPORTED + ' Path: ' + pathKey +
                ' Operation: ' + operationKey + ' Parameter: ' + parameterKey);
              //no need to return here, we want to be able to validate errors
              // in multiple parameters, and not necessarily stop on the first
            }
          });
        });
      }
    });
  });

  // Todo: load the parameters of the operations and
  // then make sure there are no JSON_REF in them
  var parameters = null;

  //validate the parameters
  if (parameters === null || parameters.length() === 0) {
    return;
  }

  for (var i = 0; i < parameters.length(); i++) {
    var field = getSwaggerFieldPath() + '.parameters[' + i + ']';
    var param = parameters[i];

    if (JSON_REF in param) {
      apimError.push(SWAGGER_JSON_REF_NOT_SUPPORTED + field);
      return;
    }
  }
}

function getSwaggerFieldPath() {
  throw new Error('The feature is under construction.');
}

function validateSecurityForAPIMLimitations(data) {
  // Todo: If needed for off line validation
}

function validateSecurityForApimLimitations(data) {
  // Todo: If needed for off line validation
}

function validateHostForNonApimLimitations(data) {
  if (_.trim(data) === null) {
    logger.error(SWAGGER_2_API_HOST_MISSING);
  }
}

function isEnforced() {
  // default 'enforced' is true
  // Todo: load vendor extensions from the yml file incase specified
  if (!vendorExtensions) {
    return true;
  }

  var configuration = vendorExtensions[SWAGGER_EXTENSION_CONFIGURATION];
  if (!configuration) {
    return true;
  }

  var isEnforcedVar = configuration[SWAGGER_EXTENSION_CONFIGURATION_ENFORCED];
  if (isEnforcedVar === null || isEnforcedVar === true) {
    return true;
  }

  return false;
}

function isRealized() {
  // default 'realized' is true
  var phase = getPhase();
  if (phase === null) {
    return true;
  }

  return phase.toLowerCase() === SWAGGER_EXTENSION_CONFIGURATION_PHASE_REALIZED;
}

function getPhase() {
  // Todo: load vendor extensions from the yaml file incase specified
  if (!vendorExtensions) {
    return null;
  }

  var configuration = vendorExtensions[SWAGGER_EXTENSION_CONFIGURATION];
  if (!configuration) {
    return null;
  }

  var phase = configuration[SWAGGER_EXTENSION_CONFIGURATION_PHASE];
  return _.trim(phase);
}

/**
 * Create a {@see Validator} instance based on {@see type}
 *
 * @param type {string} valid values {'api', 'type'}
 * @return {@see Validator}
 * @throw Error if no validate has been found
 */
module.exports.createValidator = function(/**String*/type) {
  if (!(type in VALIDATOR_TYPES)) {
    logger.writeln('Invalid validator type : ' + type);
    throw new Error(f('The validator type %s is invalid.', type));
    //return;
  }

  validatorType = type;

  if (type && validators[type]) {
    return validators[type]();
  }
  else {
    return new Validator();   //Todo: Is this a valid scenario ?
  }
};
